package jmPackage;
/**
 *  Jeanne MacIlvain
 *  ECE430 Image Processing
 *  Homework Problem 4:
 *   program adjusts image using computation of Uniform
 *      Non-Adaptive Histogram Equalization (UNAEH)
 */
import java.awt.*;

public class HistogramEQProcessor extends j2d.imageproc.ImageProcessor{
    private int lookupTable[] = new int[256];
    private double PMF[][] = new double[3][256];
    private double CMF[][] = new double[3][256];
    private double avgCMF[] = new double[256];

    // Create a new HistogramEQProcessor
    public HistogramEQProcessor(Image image){
        for (int j = 0; j< 256; j++){
            lookupTable[j] = j;
        }

        /* set base image reference to image object
              passed into method   */
        Image im = image;
        setBaseImage(im);
    }

    // Implementation of ImageProcessor
    public void processImage() throws Exception{
        int pixels[];
        int r, g, b;

        pixels = getPixels();

        for (int i = 0; i<pixels.length; i++){

            // Separate RGB components
            r = (pixels[i] & 0x00FF0000) >> 16;
            g = (pixels[i] & 0x0000FF00) >> 8;
            b = (pixels[i] & 0x000000FF);

            // Adjust the pixel
            r = lookupTable[r];
            g = lookupTable[g];
            b = lookupTable[b];

            // store the processed pixel
            pixels[i] = 0xFF000000 | (r << 16) | (g << 8) | b;
        }

        setPixels(pixels);

    }

    /**
     * Set the base image reference.  Compute aggregate statistics
     * on the image that will be needed for histogram processing
     */
    public void setBaseImage(Image newImage){
        super.setBaseImage(newImage);
        computeStats();
        // compute lookup table
        makeUNAHETable();
    }

   // Method computes PMF and CMF stats
    private void computeStats(){
        int pixels[];
        int r, g, b;
        int i, j;
        double increment;

        pixels = getPixels();

        // Zero out all the stats
        for (j = 0; j< 256; j++){
            for ( i= 0; i<3; i++){
                PMF[i][j] = 0;
                CMF[i][j] = 0;
            }
            avgCMF[j] = 0;
        }

        // Count pixels
        increment = 1.0/pixels.length;
        for (i = 0; i<pixels.length; i++){

            // Separate RGB components
            r = (pixels[i] & 0x00FF0000) >> 16;
            g = (pixels[i] & 0x0000FF00) >> 8;
            b = (pixels[i] & 0x000000FF);

            PMF[0][r] += increment;
            PMF[1][g] += increment;
            PMF[2][b] += increment;
        }
        for (i = 0; i< 3; i++){
            CMF[i][0] = PMF[i][0];
            for (j = 1; j < 256; j++){
                CMF[i][j] = CMF[i][j-1] + PMF[i][j];
            }
        }
        for (i = 0; i < 3; i++){
            for (j=0; j < 256; j++){
                avgCMF[j] += CMF[i][j] / 3;
            }
        }
    }

     /* Create lookup table for Uniform Non Adaptive
        Histogram Equalization  */
    public void makeUNAHETable() {
        for (short i = 0; i < lookupTable.length; i++){
            lookupTable[i] = (short) (255 * avgCMF[i]);
        }
    }


}
