package jmPackage;
/**
 *  Jeanne MacIlvain
 *  ECE430 Image Processing
 *  Homework Problem 4:
 *   Program inputs java.awt.Image, utilizes PMF and CMF
 *    to perform Uniform Non-Adaptive Histogram Equalization
 *    (UNAHE) for the image
 */
import j2d.ImageUtils;
import j2d.gui.ImagePanel;
import java.util.HashMap;
import java.util.Iterator;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.InputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import futils.Futil;
import futils.StreamSniffer;
import gui.ClosableJFrame;

public class ImageUNAHE {
    private static double pmf[][];
    private static ColorMapGenerator cmg;
    private static HashMap colorMap;
    private static Image image;
    static int pmf_part = 1;
    static int color_part = 0;
    static int pair_size = 2;

    public static void createColorMap(){
        // read input image file and display in gui
        image = getFileImage();
        cmg = new ColorMapGenerator(image);
        cmg.processImage();
        colorMap = cmg.getColorMap();
    }

    public static void main(String[] args) {
        createColorMap();
        calcPmf();
        quicksort(pmf, 0, pmf.length-1);

        System.out.println("Most Probable:");
        for(int i=0; i<10; i++){
            System.out.println(pmf[i][color_part] + " " + pmf[i][pmf_part]);
        }
        System.out.println("Least Probable:");
        for(int j=pmf.length-1; j>pmf.length-11; j--){
            System.out.println(pmf[j][color_part] + " " + pmf[j][pmf_part]);
        }

        // perform UNAHE on the image and display output in gui
        imageUNAHE(image);
    }

    // method uses dialog to query user for image file
    public static java.awt.Image getFileImage() {
        // dialog to open file directory to input image file
        java.io.File f =
                Futil.getReadFile("select an image file");
        if (f == null) return null;
        InputStream is;
        try {
            is = new FileInputStream(f);
            new StreamSniffer(is);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
        return getGifJpgPngImage(f);
    }
     // method displays file image read in gui jframe
     public static java.awt.Image getGifJpgPngImage(java.io.File f) {
        java.awt.Image i =
                java.awt.Toolkit.getDefaultToolkit().getImage(f.toString());
        ImageUtils.waitForImage(new gui.ClosableJFrame(), i);
        System.out.println("file image input for UNAHE computation is: " + f.getName());

        // input java.awt.Image file and place in panel
        ImagePanel ip = new ImagePanel();
        ip.setImage(i);
         
        // display image panel in JFrame
        ClosableJFrame cjf = new ClosableJFrame("Input Image");
        Container c = cjf.getContentPane();
        c.setLayout(new FlowLayout());
        c.add(ip);
        cjf.setSize(200, 200);
        cjf.setVisible(true);

        // return image
        return i;
    }

    // method performs the UNAHE for the image
    private static void imageUNAHE(java.awt.Image i) {
        BufferedImage bi;
        bi = ImageUtils.getBufferedImage(i);
        Image im = ImageUtils.getImage(bi);

        // HistogramEQProcessor class adjusts image using UNAHE
        HistogramEQProcessor heqp = new HistogramEQProcessor(im);

        try {
            heqp.processImage();
            ImagePanel ip = new ImagePanel();

            // place UNAHE processed image file in panel
            ip.setImage(heqp.getProcessedImage());

            // display output image panel in JFrame
            ClosableJFrame cjf = new ClosableJFrame("Output UNAHE Image");
            Container c = cjf.getContentPane();
            c.setLayout(new FlowLayout());
            c.add(ip);
            cjf.setSize(200, 200);
            cjf.setVisible(true);

        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    public static void calcPmf(){
        Iterator colorMapIndex = colorMap.keySet().iterator();
        pmf = new double[colorMap.size()][pair_size];
        int colorCount = 0;
        int totalPixels = cmg.getBaseImageHeight()*cmg.getBaseImageWidth();

        do {
            // Returns color as Object
            Object colorKey = colorMapIndex.next();
            // pixels per this one color
            Color color = (Color) colorKey;
            float pixelCount = Integer.parseInt(
                    colorMap.get(colorKey).toString());

            pmf[colorCount][pmf_part] = pixelCount / totalPixels;
            pmf[colorCount][color_part] = colorCount;

            colorCount++;

        } while (colorMapIndex.hasNext());

         // print total colors
          System.out.println("Total number of colors = " + colorCount);
    }

    public static void quicksort(double[][] a, int left, int right) {
        if (right <= left) return;
        int i = partition(a, left, right);
        quicksort(a, left, i-1);
        quicksort(a, i+1, right);
    }

    private static int partition(double[][] a, int left, int right) {
        int i = left - 1;
        int j = right;
        while(true) {
            // find item on left to swap, a[right] acts as sentinel
            while (more(a[++i][pmf_part], a[right][pmf_part]));
            // find item on right to swap
            while (more(a[right][pmf_part], a[--j][pmf_part]))
                if (j == left) break;   // don't go out-of-bounds
            if (i >= j) break;          // check if pointers cross
            exch(a, i, j);              // swap two elements into place
        }
        exch(a, i, right);              // swap with partition element
        return i;
    }

    // is x < y ?
    private static boolean more(double x, double y) {
        return (x > y);
    }

    // exchange a[i] and a[j]
    private static void exch(double[][] a, int i, int j) {
        double[] swap = a[i];
        a[i] = a[j];
        a[j] = swap;
    }
}
